<?php
/**
 * $Horde: wicked/display.php,v 1.22 2005/01/03 14:35:46 jan Exp $
 *
 * Copyright 2003-2005 Tyler Colbert <tyler@colberts.us>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('WICKED_BASE', dirname(__FILE__));
require_once WICKED_BASE . '/lib/base.php';

$page = &Page::getCurrentPage();
if (is_a($page, 'PEAR_Error')) {
    $notification->push(_("Internal error viewing requested page"),
                        'horde.error');
}

$actionID = Util::getFormData('actionID');
switch ($actionID) {
case 'lock':
    if (!$page->allows(WICKED_MODE_LOCKING)) {
        $notification->push(_("You are not allowed to lock this page"),
                            'horde.error');
        break;
    }
    $result = $page->lock();
    if (is_a($result, 'PEAR_Error')) {
        $notification->push(sprintf(_("Page failed to lock: %s"), $result->toString()),
                            'horde.error');
    } else {
        $notification->push(_("Page locked"), 'horde.success');
    }
    break;

case 'unlock':
    if (!$page->allows(WICKED_MODE_LOCKING)) {
        $notification->push(_("You are not allowed to unlock this page"),
                            'horde.error');
    }
    $result = $page->unlock();
    if (is_a($result, 'PEAR_Error')) {
        $notification->push(
            sprintf(_("Page failed to unlock: %s"), $result->toString()),
            'horde.error');
    } else {
        $notification->push(_("Page unlocked"), 'horde.success');
    }
    break;

case 'edit':
    if ($page->allows(WICKED_MODE_EDIT)) {
        /* Redirect to edit page. */
        $url = Util::addParameter('display.php','page','EditPage');
        $url = Util::addParameter($url,'referrer',$page->pageName());
        $url = Horde::applicationUrl($url, true);
        header('Location: ' . $url);
        exit;
    }
    $notification->push(_("You are not allowed to edit this page"),
                        'horde.error');
    break;

case 'history':
    if ($page->allows(WICKED_MODE_HISTORY)) {
        /* Redirect to history page. */
        $url = Horde::applicationUrl('history.php', true);
        $url = Util::addParameter($url, 'page', $page->pageName());
        header('Location: ' . $url);
        exit;
    }
    $notification->push(_("This page does not have a history"), 'horde.error');
    break;

case 'revert':
    $oldpage = $page;
    $page = &Page::getPage($page->pageName());
    if (!$page->allows(WICKED_MODE_EDIT)) {
        $notification->push(_("You are not allowed to edit this page"),
                            'horde.error');
        break;
    }
    $page->updateText($oldpage->getText(), 'Revert', false);
    break;

case 'remove':
    if ($page->allows(WICKED_MODE_REMOVE)) {
        $version = Util::getFormData('version');
        if (empty($version)) {
            $page->removeAllVersions();
        } else {
            $page->removeVersion($version);
        }
        /* Redirect to home page. */
        $url = Horde::applicationUrl('display.php', true);
        header('Location: ' . $url);
        exit;
    }
    $notification->push(_("This page cannot be deleted"), 'horde.error');
    break;

case 'special':
    $page->handleAction();
    break;

default:
    $wicked->logPageView($page->pageName());
}

if (!$page->allows(WICKED_MODE_DISPLAY)) {
    $notification->push(_("You don't have permission to view this page."),
                        'horde.error');
}

if (!isset($_SESSION['wicked']) || !isset($_SESSION['wicked']['history'])) {
    $_SESSION['wicked']['history'] = array();
}

$title = $page->pageName();
require WICKED_TEMPLATES . '/common-header.inc';
require WICKED_TEMPLATES . '/menu.inc';

if (!is_a($page, 'PEAR_Error')) {
    if ($page->allows(WICKED_MODE_DISPLAY)) {
        $page->render(WICKED_MODE_DISPLAY, Util::getFormData('params'));
    }
}

require $registry->get('templates', 'horde') . '/common-footer.inc';

if (!isset($_SESSION['wicked']['history'][0]) ||
    $_SESSION['wicked']['history'][0] != $page->pageName()) {
    array_unshift($_SESSION['wicked']['history'], $page->pageName());
}
if (count($_SESSION['wicked']['history']) > 10) {
    array_pop($_SESSION['wicked']['history']);
}
