<?php

require_once WICKED_BASE . '/lib/Page/StandardPage.php';

/**
 * Special page for merging or renaming pages.
 *
 * $Horde: wicked/lib/Page/MergeOrRename.php,v 1.7 2004/12/01 16:28:08 eraserhd Exp $
 *
 * @author  Jason M. Felice <eraserhd@speakeasy.net>
 * @version $Revision: 1.7 $
 * @package Wicked
 */
class MergeOrRename extends Page {

    /**
     * Display modes supported by this page.
     * @var array $supportedModes
     */
    var $supportedModes = array(
        WICKED_MODE_DISPLAY => true);

    /**
     * The page that we're displaying similar pages to.
     * @var string $_referrer
     */
    var $_referrer = null;

    /**
     * Validation errors.
     *
     * @var string $_errors
     */
    var $_errors = array();

    function MergeOrRename($referrer)
    {
        $this->_referrer = $referrer;
    }

    /** 
     * Retrieve this user's permissions for the referring page.
     *
     * @return integer  The permissions bitmask.
     */
    function getPermissions()
    {
        $page = &Page::getPage($this->referrer(), null, null, null, null);
        return $page->getPermissions();
    }

    /**
     * Render this page in Display mode.
     *
     * @return mixed                Returns true or PEAR_Error.
     */
    function display()
    {
        global $wicked, $registry;

        require_once 'Horde/Template.php';
        $template = &new Horde_Template();
        $template->setOption('gettext', true);

        $referrer = $this->referrer();
        $template->set('pagename', 'MergeOrRename');
        $template->set('referrer', $referrer);

        $referrerLink = Util::addParameter(Horde::applicationUrl('display.php'),
                                           array('page' => $referrer));
        $template->set('referrerLink', $referrerLink);

        $refreshLink = Horde::link($this->pageUrl('display.php')) .
                       Horde::img('reload.png',
                                  sprintf(_("Reload %s"), $this->pageName()),
                                  '',
                                  $GLOBALS['registry']->getImageDir('horde')) .
                       '</a>';
        $template->set('refreshLink', $refreshLink);

        $requiredMarker = Horde::img('required.png', '*', '',
                                     $registry->getImageDir('horde'));
        $template->set('requiredMarker', $requiredMarker);

        $references = $wicked->getBackLinks($referrer);
        if (is_a($references, 'PEAR_Error')){
            global $notification;
            $notification->push('Error retrieving back links: ' .
                                $references->toString(), 'horde.error');
            return $references;
        }

        $item0 = true;
        foreach ($references as $key => $page) {
            $references[$key]['class'] = $item0 ? 'item0' : 'item1';
            $item0 = !$item0;

            $url = Util::addParameter(Horde::applicationUrl('display.php'),
                                      array('page' => $page['page_name']));
            $references[$key]['page_url'] = @htmlspecialchars($url, ENT_QUOTES, NLS::getCharset());

            $references[$key]['page_name'] = @htmlspecialchars($page['page_name'], ENT_QUOTES, NLS::getCharset());

            // Since the page name can have [ and ] and other special
            // characters in it, and we don't want the browser or PHP decoding
            // it, we encode it in quoted printable for the checkbox names.
            $references[$key]['checkbox'] = preg_replace('/([^a-zA-Z_0-9 ])/e', '"=" . str_pad(dechex(ord(\'\\1\')), 2, \'0\', STR_PAD_LEFT)', $page['page_name']);
        }

        $template->set('references', $references);
        $template->set('referenceCount', sprintf(_("This page is referenced from %d other page(s)."), count($references)));
        $template->set('formInput', Util::formInput());

        // Propogate any validation errors.
        foreach (array('new_name', 'collision') as $elt) {
            if (!isset($this->_errors[$elt])) {
                $this->_errors[$elt] = '';
            }
        }
        $template->set('errors', $this->_errors);

        $template->set('new_name', Util::getFormData('new_name'));

        echo $template->fetch(WICKED_TEMPLATES . '/display/MergeOrRename.html');
        return true;
    }

    function pageName()
    {
        return sprintf(_("MergeOrRename: %s"), $this->referrer());
    }

    function referrer()
    {
        return $this->_referrer;
    }

    function pageUrl($linkpage = 'display.php', $actionid = 'none')
    {
        $url = Horde::applicationUrl($linkpage);
        $url = Util::addParameter($url, 'page', $this->pageName());
        $url = Util::addParameter($url, 'referrer', $this->referrer());
        return Util::addParameter($url, 'actionID', $actionid);
    }

    /**
     * Retrieve the form fields and process the merge or rename.
     */
    function handleAction()
    {
        $referrer = $this->referrer();

        $new_name = Util::getFormData('new_name');
        if (empty($new_name)) {
            $this->_errors['new_name'] = _("This is a required field.");
        } elseif ($new_name == $referrer) {
            $this->_errors['new_name'] = _("New name is the same as old name.");
        }
        $collision = Util::getFormData('collision');
        if (empty($collision)) {
            $this->_errors['collision'] = _("This is a required field.");
        }

        if (count($this->_errors)) {
            return;
        }

        global $wicked, $notification;
        $srcPage = $wicked->retrieveByName($referrer);
        if (is_a($srcPage, 'PEAR_Error')) {
            $notification->push(sprintf(_("Failed to retrieve '%s': %s"),
                                        $referrer, $srcPage->toString()),
                                'horde.error');
            return;
        }

        $dstPage = $wicked->retrieveByName($new_name);
        if (!is_a($dstPage, 'PEAR_Error')) {
            if ($collision != 'merge') {
                // fail
                $notification->push(sprintf(_("Page '%s' already exists."),
                                            $new_name), 'horde.error');
                return;
            }

            $newText = $dstPage['page_text'] . "\n----\n" .
                       $srcPage['page_text'];
            $changelog = sprintf(_("Merged from %s"), $referrer);
            $res = $wicked->updateText($new_name, $newText, $changelog, true);
            if (is_a($res, 'PEAR_Error')) {
                $notification->push(sprintf(_("Error updating %s: %s"),
                                            $new_name, $res->toString()),
                                    'horde.error');
                return;
            }

            $res = $wicked->removeAllVersions($referrer);
            if (is_a($res, 'PEAR_Error')) {
                $notification->push(sprintf(_("Error deleting %s: %s"),
                                            $referrer, $res->toString()),
                                    'horde.error');
            }
        } else {
            $res = $wicked->renamePage($referrer, $new_name);
            if (is_a($res, 'PEAR_Error')) {
                $notification->push(sprintf(_("Error renaming %s: %s"),
                                            $referrer, $res->toString()),
                                    'horde.error');
                return;
            }
        }

        $references = Util::getFormData('ref');
        if (empty($references)) {
            return;
        }

        $wikiWord = '/^' . WICKED_REGEXP_WIKIWORD . '$/';

        foreach ($references as $name => $value) {
            $page_name = quoted_printable_decode($name);

            // Fix up for self-references.
            if ($page_name == $referrer) {
                $page_name = $new_name;
            }

            $refPage = $wicked->retrieveByName($page_name);
            if (is_a($refPage, 'PEAR_Error')) {
                $notification->push(sprintf(_("Error retrieving %s: %s"),
                                            $page_name, $refPage->toString()),
                                    'horde.error');
                continue;
            }

            $changelog = sprintf(_("Changed references from %s to %s"),
                                 $referrer, $new_name); 

            if (preg_match($wikiWord, $new_name)) {
                $replaceWith = $new_name;
            } else {
                $replaceWith = '((' . $new_name . '))';
            }

            $from = array('/\(\(' . preg_quote($referrer, '/') . '\)\)/');
            $to = array($replaceWith);

            // If this works as a bare wiki word, replace that, too.
            if (preg_match($wikiWord, $referrer)) {
                $from[] = '/\b' . preg_quote($referrer, '/') . '\b/';
                $to[] = $replaceWith;
            }

            $newText = preg_replace($from, $to, $refPage['page_text']);
            $res = $wicked->updateText($page_name, $newText, $changelog, true);
            if (is_a($res, 'PEAR_Error')) {
                $notification->push(sprintf(_("Error updating %s: %s"),
                                            $page_name, $res->toString()),
                                    'horde.warning');
            }
        }

        $url = Util::addParameter(Horde::applicationUrl('display.php'),
                                  array('page' => $new_name));
        header("Location: " . $url);
        exit;
    }

}
